@@ -1,12 +1,12 @@
 import csv
 import json
-import os
+from os import path
 from datetime import datetime
 
 from openpyxl import load_workbook
 
 from resources import excel_headers
-
+from excel2meta_interface.utils import messages
 
 class Excel2Reference:
     start_time = datetime.now()
@@ -16,7 +16,7 @@ class Excel2Reference:
         self.excel_sheet = "excel format"
         self.json_directory = "out/"
         self.json_file_prefix = "generated_"
-        self.result = 0
+        self.result = messages.message["ok"]
         self.work_book = None
         self.work_sheet = None
 
@@ -25,7 +25,8 @@ class Excel2Reference:
                 data = json.load(config)
                 self.excel_location = data["excel_location"]
                 self.excel_sheet = data["excel_sheet"]
-                self.json_directory = data["json_directory"]
+                if "json_directory" in data:
+                    self.json_directory = data["json_directory"]
                 if "json_file_prefix" in data:
                     self.json_file_prefix = data["json_file_prefix"]
                 else:
@@ -35,7 +36,7 @@ class Excel2Reference:
                 self.json_file_prefix += "--"
         except FileNotFoundError:
             print("Configuration file >" + configuration_file + "< not found.")
-            self.result = 1
+            self.result = messages.message["main_config_not_found"]
 
     def read_excel(self, file=None, sheet=None):
         """
@@ -47,37 +48,42 @@ class Excel2Reference:
         :param sheet: The Excel sheet in the file to be loaded
         :return: result = 2 (one or more parameters not provided (None)), 3 (FileNotFound), 10 (IOError), 0 = ok
         """
-        path = ""
-        result = 0
+        result = messages.message["ok"]
+        excel_path = ""
         if file is None or sheet is None:
-            return 2
-        self.work_book = None
-        self.work_sheet = None
+            result = messages.message["internal_error"]
+            result["info"] = "read_excel was called without an Excel file name >%s< and/or without a sheet name >%s<." \
+                             % (file, sheet)
+            return result
+
         try:
-            path = self.excel_location + "/" + file
-            self.work_book = load_workbook(path)
+            excel_path = path.join(self.excel_location, file)
+            self.work_book = load_workbook(excel_path)
             self.work_sheet = self.work_book[sheet]
-        except FileNotFoundError as e:
-            print("Could not find/read Excel >" + path + "<. Error: " + str(e))
-            return 3
-        except IOError as e:
-            print("IOError loading Excel file >" + path + "<. Error: " + str(e))
-            return 10
+        except FileNotFoundError or IOError as e:
+            result = messages.message["excel_not_found"]
+            result["info"] = "Could not find/read Excel >" + excel_path + "<. Error: " + str(e)
+            print(result["info"])
+            return result
 
         return result
 
     def create_output_files(self):
-        result = 0
+        overall_result = messages.message["ok"]
         info_datasets = []
         # list of source zones
         source_zone_column = "zone"
         source_zones = self.get_distinct(source_zone_column)
-        result += self.out_distinct("REF_-_zones-source" + source_zone_column, source_zones)
+        result = self.out_distinct("REF_-_zones-source" + source_zone_column, source_zones)
+        if result["code"] != "OK":
+            overall_result = result
 
         # list of target zones
         target_zone_column = "tgtzone"
         target_zones = self.get_distinct(target_zone_column)
-        result += self.out_distinct("REF_-_zones-target" + target_zone_column, target_zones)
+        result = self.out_distinct("REF_-_zones-target" + target_zone_column, target_zones)
+        if result["code"] != "OK":
+            overall_result = result
 
         # source dataset list
         source_dataset_column = "dataset"
@@ -86,7 +92,9 @@ class Excel2Reference:
         info_source_datasets = self.get_dataset_info(source_datasets, "dataset")
         print("dataset info collected for >%d< source datasets" % len(info_source_datasets))
         info_datasets.extend(info_source_datasets)
-        result += self.out_distinct_combined("REF_-_data-sources-source" + source_dataset_column, info_source_datasets)
+        result = self.out_distinct_combined("REF_-_data-sources-source" + source_dataset_column, info_source_datasets)
+        if result["code"] != "OK":
+            overall_result = result
 
         # target dataset list
         target_dataset_column = "tgtdataset"
@@ -96,14 +104,18 @@ class Excel2Reference:
         print("dataset info collected for >%d< target datasets" % len(info_target_datasets))
         info_datasets.extend(info_target_datasets)
         print("In total: dataset info collected for >%d< datasets" % len(info_datasets))
-        result += self.out_distinct_combined("REF_-_data-sources-target" + target_dataset_column, info_target_datasets)
+        result = self.out_distinct_combined("REF_-_data-sources-target" + target_dataset_column, info_target_datasets)
+        if result["code"] != "OK":
+            overall_result = result
 
         # which source zone has which target zone
         src_zone, row = self.search_value_in_row_index(search_string=source_zone_column, row=1)
         tgt_zone, row = self.search_value_in_row_index(search_string=target_zone_column, row=1)
         src_tgt_zones = self.get_distinct_combined(src_zone, tgt_zone)
-        result += self.out_distinct_combined("MAP_-_zones-source2target"
+        result = self.out_distinct_combined("MAP_-_zones-source2target"
                                              + source_zone_column + '_' + target_zone_column, src_tgt_zones)
+        if result["code"] != "OK":
+            overall_result = result
 
         # which source dataset in which source zone has which target dataset in which target zone
         src_zone, row = self.search_value_in_row_index(search_string=source_zone_column, row=1)
@@ -115,10 +127,13 @@ class Excel2Reference:
                                                             , source_dataset=src_dataset
                                                             , target_dataset=tgt_dataset
                                                             )
-        result += self.out_distinct_combined("MAP_-_data-sources-source2target"
+        result = self.out_distinct_combined("MAP_-_data-sources-source2target"
                                              + source_zone_column + "." + source_dataset_column
                                              + "_" + target_zone_column + "." + target_dataset_column
                                              , src_tgt_zones_datasets)
+        if result["code"] != "OK":
+            overall_result = result
+
         for dataset in info_datasets:
             entities = []
             attributes = self.get_dataset_attributes(dataset_info=dataset)
@@ -127,10 +142,10 @@ class Excel2Reference:
                           , src_zone=dataset["src_zone"]
                           , attributes=attributes)
             entities.append(entity)
-            file_name = os.path.join(self.json_directory, self.json_file_prefix + "ATTRIB_-_" + dataset["dataset"])
+            file_name = path.join(self.json_directory, self.json_file_prefix + "ATTRIB_-_" + dataset["dataset"])
             self.write_dict_to_file(out_file=file_name, result_list=entities)
 
-        return result
+        return overall_result
 
     def get_dataset_attributes(self, dataset_info):
         # print(dataset_info)
@@ -313,7 +328,7 @@ class Excel2Reference:
         if column_name is None or result_list is None:
             return None
         print(result_list)
-        out_file = os.path.join(self.json_directory, self.json_file_prefix + column_name + ".csv")
+        out_file = path.join(self.json_directory, self.json_file_prefix + column_name + ".csv")
         return self.write_list_to_file(out_file, result_list)
 
     def out_distinct_combined(self, filename, source_targets=None):
@@ -321,7 +336,7 @@ class Excel2Reference:
             return 6
         print(source_targets)
         # method will add extension
-        out_file = os.path.join(self.json_directory, self.json_file_prefix + filename)
+        out_file = path.join(self.json_directory, self.json_file_prefix + filename)
         return self.write_dict_to_file(out_file, source_targets)
 
     def get_distinct(self, column_name=None):
@@ -412,10 +427,11 @@ class Excel2Reference:
                         csv_file.write(item.replace('/', ';') + "\n")
         except IOError:
             print("Error creating or writing to output file >" + out_file + "<.")
-            return 4
-        return 0
+            return messages.message["write_error"]
+        return messages.message["ok"]
 
     def write_dict_to_file(self, out_file, result_list):
+        result = messages.message["ok"]
         # csv
         file_name = out_file + ".csv"
         try:
@@ -425,24 +441,27 @@ class Excel2Reference:
                 dict_writer.writeheader()
                 dict_writer.writerows(result_list)
         except IOError:
-            print("Error creating or writing dictionary to csv output file >" + file_name + "<.")
-            return 8
+            result = messages.message["write_error"]
+            result["info"] = "Error creating or writing dictionary to csv output file >" + file_name + "<."
+            return result
         # json
         file_name = out_file + ".json"
         try:
             with open(file_name, "w", encoding="utf8", newline='') as json_file:
                 json_file.write(json.dumps(result_list, indent=4))
         except IOError:
-            print("Error creating or writing dictionary to json output file >" + file_name + "<.")
-            return 9
-        return 0
+            result = messages.message["write_error"]
+            result["info"] = "Error creating or writing dictionary to json output file >" + file_name + "<."
+            print(result["info"])
+
+        return result
 
     def main(self, excel=None):
-        if self.result == 1:
+        if self.result["code"] != "OK":
             return self.result
 
         result = self.read_excel(excel, self.excel_sheet)
-        if result != 0:
+        if result["code"] != "OK":
             return result
 
         result = self.create_output_files()
@@ -452,4 +471,8 @@ class Excel2Reference:
 if __name__ == "__main__":
     excel2ref = Excel2Reference(configuration_file="resources/excel2json_config.json")
     result = excel2ref.main(excel="tryout.xlsx")
-    exit(result)
+    print("result:", result)
+    if result["code"] != "OK":
+        exit(1)
+    else:
+        exit(0)
